/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "hexmanagerwidget.h"

CHexManagerWidget::CHexManagerWidget( QWidget *parent, const char *name,
				      bool showState )
  : QWidget( parent, name )
{
  mValid = false;

  mEditor = new CHexEditorWidget( this );
  if( mEditor == 0 ) { return; }
  
  mUtil = new CHexToolWidget( this );
  if( mUtil == 0 ) { return; }

  connect( mEditor->view(), SIGNAL( cursorChanged( SCursorState & ) ),
	   mUtil, SLOT( cursorChanged( SCursorState & ) ) );

  mValid      = true;
  mUtilHeight = 150;

  setShowUtil( showState );
  show();
}

CHexManagerWidget::~CHexManagerWidget( void )
{
  delete mEditor;
  delete mUtil;
}


void CHexManagerWidget::resizeEvent( QResizeEvent *e )
{
  mUtilHeight = mUtil->minimumSize().height();
  

  if( mShowUtil == true )
  {
    mEditor->setGeometry( 0, 0, width(), height() - mUtilHeight );
    mUtil->setGeometry( 0, height() - mUtilHeight, width(), mUtilHeight );
  }
  else
  {
    mEditor->setGeometry( 0, 0, width(), height() );
  }
}


void CHexManagerWidget::setShowUtil( bool state )
{
  if( mValid == false ) { return; }

  mUtilHeight = mUtil->minimumSize().height();

  mShowUtil = state;
  if( mShowUtil == true )
  {
    mEditor->setGeometry( 0, 0, width(), height() - mUtilHeight );
    mUtil->setGeometry( 0, height() - mUtilHeight, width(), mUtilHeight );
    mUtil->show();
  }
  else
  {
    mEditor->setGeometry( 0, 0, width(), height() );
    mUtil->hide();
  }
}





