/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef _OPTION_DIALOG_H_
#define _OPTION_DIALOG_H_

#include <qtabdlg.h>
#include <qframe.h>
#include <qlined.h>
#include <qcombo.h>
#include <qchkbox.h>
#include <kconfig.h>
#include <qradiobt.h>
#include <qpushbt.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qlayout.h> 

#include <kcolorbtn.h>
#include <ktabctl.h>

#include "pagedialog.h"
#include "fontchooser.h"
#include "hexeditstate.h"


class CLineSizeWidget : public KTabCtl
{
  Q_OBJECT

  public:
    inline CLineSizeWidget( QWidget *parent = 0, const char *name = 0 );
    inline ~CLineSizeWidget( void );

    inline void setValue( const SDisplayLine &line );
    inline void getValue( SDisplayLine &line );

  protected:

  public slots:

  private:
  
  private:
    QFrame *page[4];
    QSpinBox *lineSpin[4];
    QSpinBox *columnSpin[4];
};


inline CLineSizeWidget::CLineSizeWidget( QWidget *parent, const char *name )
  :KTabCtl( parent, name )
{
  const char *text;
  setBorder( false );

  const char *formatStrings[] = 
  {
    i18n("Hexadecimal"),
    i18n("Octal"),
    i18n("Binary"),
    i18n("Text only")
  };

  for( uint i=0; i<4;i++)
  {
    page[i] = new QFrame( this );
    if( page[i]  == 0 ) { return; }
    page[i]->setFrameStyle( QFrame::Box|QFrame::Sunken );
    addTab( page[i], formatStrings[i] ); 
    
    QGridLayout *gbox = new QGridLayout( page[i], 2, 2, 5 );
    lineSpin[i] = new QSpinBox( page[i] );
    lineSpin[i]->setFixedHeight( lineSpin[i]->sizeHint().height() );
    lineSpin[i]->setRange( 1, 1000 );
    gbox->addWidget( lineSpin[i], 0, 1 );
    columnSpin[i] = new QSpinBox( page[i] );
    columnSpin[i]->setFixedHeight( columnSpin[i]->sizeHint().height() );
    columnSpin[i]->setRange( 1, 1000 );
    gbox->addWidget( columnSpin[i], 1, 1 );
    text = i18n("Default li&ne size [bytes]");
    QLabel *label1 = new QLabel( lineSpin[i], text, page[i] );
    gbox->addWidget( label1, 0, 0 );
    text = i18n("Column &size");
    QLabel *label2 = new QLabel(columnSpin[i], text, page[i] );
    gbox->addWidget( label2, 1, 0 );
    gbox->activate();
  }

  connect( lineSpin[3], SIGNAL(valueChanged (int)),
	   columnSpin[3], SLOT(setValue (int)) );
  connect( columnSpin[3], SIGNAL(valueChanged (int)),
	   lineSpin[3], SLOT(setValue (int)) );




  SDisplayLine line;
  setValue( line );

  show();
}

inline CLineSizeWidget::~CLineSizeWidget( void )
{
}

inline void CLineSizeWidget::setValue( const SDisplayLine &line )
{
  for( uint i=0; i<4; i++)
  {
    lineSpin[i]->setValue(line.lineSize[i]);
    columnSpin[i]->setValue(line.columnSize[i]);
  }
}

inline void CLineSizeWidget::getValue( SDisplayLine &line )
{
  for( uint i=0; i<4; i++)
  {
    line.lineSize[i] = lineSpin[i]->value();
    line.columnSize[i] = columnSpin[i]->value();
  }
}



class COptionDialog : public CPageDialog
{
  Q_OBJECT

  public:
    enum EPage
    {
      page_layout = 0,
      page_cursor,
      page_color,
      page_font,
      page_misc,
      page_max,
    };

    COptionDialog( const char *title, 
		   QWidget *parent = 0, char *name = 0, bool modal = FALSE );
    ~COptionDialog( void );

    void setLayout( SDisplayLayout &layout, SDisplayLine &line );
    void setCursor( SDisplayCursor &cursor );
    void setColor( SDisplayColor &color );
    void setFont( SDisplayFont &font );
    void setMisc( SDisplayMisc &misc );
    void setState( SDisplayState &state );

  public slots:

  signals:
    void lineSizeChoise( const SDisplayLine &lineSize );
    void layoutChoise( const SDisplayLayout &layout );
    void cursorChoise( const SDisplayCursor &cursor ); 
    void colorChoise( const SDisplayColor &color );
    void fontChoise( const SDisplayFont &font );
    void miscChoise( const SDisplayMisc &misc );

  protected slots:
    void slotModeSelectorChanged( int index );
    void slotLineSizeChanged( int value );
    void slotColumnSizeChanged( int value );
    void slotColumnSepCheck( bool state );
    void slotBlinkIntervalCheck( bool state );
    void slotBlockCursorCheck( bool state );
    void slotFontSystem( bool state );
    void slotColorSystem( bool state );
    void slotThresholdConfirm( void );
    void slotDefault( void );
    void slotApply( void );

  protected:

  private:
    struct SLayoutWidgets
    {
      QComboBox *formatCombo;
      QSpinBox  *lineSizeSpin;
      QSpinBox  *columnSizeSpin;
      QCheckBox *lockLineCheck;
      QCheckBox *lockColumnCheck;
      QCheckBox *columnCheck;
      QLabel    *leftSepLabel;
      QLabel    *rightSepLabel;
      QSpinBox  *leftSepWidthSpin;
      QSpinBox  *rightSepWidthSpin;
      QSpinBox  *separatorSpin;
      QSpinBox  *edgeSpin;
      QLabel    *columnSepLabel;
      QSpinBox  *columnSepSpin;

    };
    struct SCursorWidgets 
    {
      QCheckBox *blinkCheck;
      QSpinBox  *blinkSpin;
      QLabel    *blinkLabel;  
      QCheckBox *blockCheck;
      QCheckBox *thickCheck;
      QRadioButton *stopRadio;
      QRadioButton *hideRadio;
      QRadioButton *nothingRadio;
    };
    struct SColorWidgets
    {
      QCheckBox    *checkSystem;
      KColorButton *textBg;
      KColorButton *inactiveBg;
      KColorButton *evenFg;
      KColorButton *oddFg;
      KColorButton *offsetFg;
      KColorButton *secondaryFg;
      KColorButton *markBg;
      KColorButton *markFg;
      KColorButton *cursorFg;
      KColorButton *cursorBg;
      KColorButton *sepFg;
      KColorButton *nonPrintFg;
      QLabel *textLabelBg;
      QLabel *inactiveLabelBg;
      QLabel *evenLabelFg;
      QLabel *oddLabelFg;
      QLabel *offsetLabelFg;
      QLabel *secondaryLabelFg;
      QLabel *markLabelBg;
      QLabel *markLabelFg;
      QLabel *cursorLabelFg;
      QLabel *cursorLabelBg;
      QLabel *sepLabelFg;
      QLabel *nonPrintLabelFg;
    };
    struct SFontWidgets
    {
      QCheckBox    *checkSystem;
      KFontChooser *chooser;
      QLineEdit    *nonPrintInput;
    };
    struct SMiscWidgets
    {
      QSpinBox  *undoSpin;
      QComboBox *openCombo;
      QCheckBox *inputCheck;
      QCheckBox *fatalCheck;
      QCheckBox *autoCheck;
      QCheckBox *insertCheck;
      QCheckBox *confirmWrapCheck;
      QCheckBox *cursorJumpCheck;
      QCheckBox *writeProtectCheck;
      QCheckBox *backupCheck;
      QCheckBox *thresholdCheck;
      QLabel    *thresholdLabel;
      QSpinBox  *thresholdSpin;
    };

  private:
    void SetupLayoutPage( void );
    void SetupCursorPage( void );
    void SetupColorPage( void );
    void SetupFontPage( void);
    void SetupMiscPage( void );
    SDisplayCursor::EFocusMode cursorFocusMode( void );

  private:
    SDisplayState  mDisplayState;
    QFrame *mFrame[ page_max ];
    SLayoutWidgets mLayout;
    SCursorWidgets mCursor;
    SColorWidgets  mColor;
    SFontWidgets   mFont;
    SMiscWidgets   mMisc;


};





#endif







