/*
 *   khexedit - Versatile hex editor
 *   Copyright (C) 1999  Espen Sand, espensa@online.no
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <iostream.h>

#include <qmsgbox.h> 
#include <kapp.h>
#include <kiconloader.h>
#include <kstdaccel.h>

#include "toplevel.h"


static char *blurb = "KHexEdit 0.7 - Versatile hex editor for KDE.\n\n"
                     "Author: Espen Sand\n" 
                     "espensa@online.no\n\n"
                     "This is a work in progress. If you\n\n"
                     "1) have any suggestions for improvement\n"
                     "2) have found a bug\n"
                     "3) just want to tell me how nice it is\n\n"
                     "then feel free to send me a mail.\n\n"
                     "Latest version is in KDE CVS and on\n"
                     "http://home.sol.no/~espensa/khexedit.";


QList<KHexEdit> KHexEdit::mWindowList;

KHexEdit::KHexEdit( void )
  : mStartupFileList( TRUE )
{
  mWindowList.append( this );
  mDocumentList.setAutoDelete( TRUE );
  mValid = false;
  mBlockRecentUpdate = false;

  //
  // Create main widget and register it.
  //
  mManager = new CHexManagerWidget( this, "manager" );
  if( mManager == 0 ) { return; }
  setView( mManager );


  connect( textView(), SIGNAL( cursorChanged( SCursorState & ) ),
	   this, SLOT( cursorChanged( SCursorState & ) ) );
  connect( textView(), SIGNAL( editMode( CHexBuffer::EEditMode ) ),
	   this, SLOT( editMode( CHexBuffer::EEditMode ) ) );
  connect( textView(), SIGNAL( encodingChanged( const SEncodeState &)),
	   this, SLOT( encodingChanged( const SEncodeState & )) );
  connect( textView(), SIGNAL( textWidth( uint ) ),
	   this, SLOT( textWidth( uint ) ) );
  connect( textView(), SIGNAL( fileModified( SFileState & ) ),
	   this, SLOT( fileModified( SFileState & ) ) );
  connect( textView(), SIGNAL( layoutChanged( const SDisplayLayout & ) ),
	   this, SLOT( layoutChanged( const SDisplayLayout & ) ) );
  connect( textView(), SIGNAL( inputModeChanged( const SDisplayInputMode & ) ),
	   this, SLOT( inputModeChanged( const SDisplayInputMode & ) ) );
  connect( textView(), SIGNAL( bookmarkChanged( QList<SCursorOffset> &)),
	   this, SLOT( bookmarkChanged( QList<SCursorOffset> & ) ) );
  connect( textView(), SIGNAL( fileName( QString & ) ),
	   this, SLOT( fileActive( QString & ) ) );
  connect( textView(), SIGNAL( fileRename( QString &, QString & ) ),
	   this, SLOT( fileRename( QString &, QString & ) ) );
  connect( textView(), SIGNAL( fileClosed( QString & ) ),
	   this, SLOT( fileClosed( QString & ) ) ); 
  connect( editor(), SIGNAL( errorLoadFile( const QString & ) ),
	   this, SLOT( removeRecentFile( const QString & ) ) );
  connect( editor(), SIGNAL( operationChanged( bool ) ),
	   this, SLOT( operationChanged( bool ) ) );
 
  //
  // The entire window will act as a drop zone (mmmm, this is really 
  // cumbersome in Motif)
  //
  mZone = new KDNDDropZone( this, DndURL );
  if( mZone == 0 ) { return; }
  connect( mZone, SIGNAL( dropAction(KDNDDropZone *) ), 
	   this, SLOT( dropAction(KDNDDropZone *) ) );


  //
  // Make the editor popup menu.
  //
  mEditorPopup = new QPopupMenu;
  if( mEditorPopup == 0 ) { return; }
  connect( mEditorPopup, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));
  mBookmarkPopup = new QPopupMenu;
  if( mBookmarkPopup == 0 ) { return; }
  connect( mBookmarkPopup, SIGNAL(activated (int)),SLOT(bookmarkPopupCB(int)));


  mEditorPopup->setCheckable( TRUE );
  mEditorPopup->insertItem( i18n("&Open..."), editor(), SLOT(open()) );
  mEditorPopup->insertItem( i18n("&Save"), editor(), SLOT(save()) );
  mEditorPopup->insertItem( i18n("Save &as..."), editor(), SLOT(saveAs()) );
  mEditorPopup->insertSeparator();
  mEditorPopup->insertItem( i18n("&Bookmarks"), mBookmarkPopup, 0, 4 );
  mEditorPopup->insertSeparator();
  mPopHexMB = mEditorPopup->insertItem( i18n("&Hexadecimal"),view_hexadecimal);
  mPopOctalMB = mEditorPopup->insertItem( i18n("&Octal"), view_octal );
  mPopBinaryMB = mEditorPopup->insertItem( i18n("&Binary"), view_binary );
  mPopTextOnlyMB = mEditorPopup->insertItem( i18n("&Text only"), view_text );
  mEditorPopup->insertSeparator();
  mPopReadOnlyMB = mEditorPopup->insertItem( i18n("&Read only"),
					     permission_readonly );
  mPopResizeMB = mEditorPopup->insertItem( i18n("&Allow resize"),
					   permission_allowResize );
  textView()->setPopupMenu( mEditorPopup );


  //
  // Make the menu bar entries. We use the default entry for the help menu.
  //
  mRecent = new QPopupMenu;
  if( mRecent == 0 ) { return; }
  connect( mRecent, SIGNAL(activated(int)), SLOT(recentFile(int)) );

  mFilePermission = new QPopupMenu;
  if( mFilePermission == 0 ) { return; }

  mFile = new QPopupMenu;
  if( mFile == 0 ) { return; }

  mEditSpecial = new QPopupMenu;
  if( mEditSpecial == 0 ) { return; }

  mEdit = new QPopupMenu;
  if( mEdit == 0 ) { return; }

  mEncoding = new QPopupMenu;
  if( mEncoding == 0 ) { return; }

  mView = new QPopupMenu;
  if( mView == 0 ) { return; }

  mBookmark = new QPopupMenu;
  if( mBookmark == 0 ) { return; }

  mDocuments = new QPopupMenu;
  if( mDocuments == 0 ) { return; }
  connect( mDocuments, SIGNAL(activated(int)), SLOT(documentMenuCB(int)) );

  mTools = new QPopupMenu;
  if( mTools == 0 ) { return; }

  mOptions = new QPopupMenu;
  if( mOptions == 0 ) { return; }

  mHelp = kapp->getHelpMenu( true, blurb );

  //
  // Make the contents of the menus a connect the entries to a slot
  //
  KStdAccel *keys = new KStdAccel( kapp->getConfig() );
  if( keys == 0 ) { return; }

  mFilePermission->setCheckable( TRUE );
  mPermissionReadOnlyMB = mFilePermission->insertItem( i18n("&Read only"), 
		      permission_readonly );
  mPermissionResizeMB = mFilePermission->insertItem( i18n("&Allow resize"), 
		      permission_allowResize );
  connect( mFilePermission, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));

  mEncoding->setCheckable( TRUE );
  mEncodingDefaultMB = mEncoding->insertItem( i18n("&Default"), 
		       encoding_default );
  mEncodingUsAsciiMB = mEncoding->insertItem( i18n("us-&ascii"),
		       encoding_us_ascii );
  mEncodingEbcdicMB  = mEncoding->insertItem( i18n("&EBCDIC (CSCP 1047)"), 
		       encoding_ebcdic );
  connect( mEncoding, SIGNAL(activated (int)), SLOT( encodingMenuCB (int) ));
  mEncoding->insertSeparator();
  mEncoding->insertItem( i18n("&User defined..."),
			 editor(), SLOT(encoding()) );

  mEncoding->setItemEnabled( mEncodingUsAsciiMB, false );

  
  mFile->insertItem( i18n("&New Document"), 
		     editor(), SLOT(newFile()), keys->openNew() );
  mFile->insertItem( i18n("&Open..."), 
		     editor(), SLOT(open()), keys->open() );
  mFile->insertItem( i18n("&Insert..."), 
		     editor(), SLOT(insertFile()), CTRL+Key_I );
  mFile->insertItem( i18n("Open R&ecent"), mRecent );
  mFile->insertItem( i18n("&Reload"), 
		     editor(), SLOT(reload()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Save"), 
		     editor(), SLOT(save()), keys->save() );
  mFile->insertItem( i18n("Save &as..."), 
		     editor(), SLOT(saveAs()), CTRL+Key_A );
  mFile->insertItem( i18n("&Close Document"), 
		     editor(), SLOT(close()), keys->close() );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Print..."), 
		     editor(), SLOT(print()), keys->print() );
  mFile->insertItem( i18n("E&xport..."), 
		     editor(), SLOT(export()) );
  mFile->insertSeparator();
  mFileCancelPrintMB = mFile->insertItem( i18n("&Cancel operation"), 
		     editor(), SLOT(stop()) );
  mFile->insertItem( i18n("Per&missions"), mFilePermission );
  mFile->insertSeparator();
  mFile->insertItem( i18n("N&ew Window"), 
		     this, SLOT(newWindow()) );
  mFile->insertItem( i18n("Close &Window"), 
		     this, SLOT(closeWindow()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Quit"), 
		     this, SLOT(closeProgram()), keys->quit() );

  mEditUndoMB = mEdit->insertItem( i18n("&Undo"), 
		     editor(), SLOT(undo()), keys->undo() );
  mEditRedoMB = mEdit->insertItem( i18n("&Redo"), 
		     editor(), SLOT(redo()), CTRL+Key_Y );
  mEdit->insertSeparator();
  mEditCopyMB = mEdit->insertItem( i18n("&Copy"),
		     editor(), SLOT(copy()), keys->copy() );
  mEdit->insertItem( i18n("&Paste"),
		     editor(), SLOT(paste()), keys->paste() );
  mEditCutMB = mEdit->insertItem( i18n("Cu&t"),
		     editor(), SLOT(cut()), keys->cut() );
  mEdit->insertItem( i18n("Spec&ial"), mEditSpecial );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Select all"), 
		     editor(), SLOT(selectAll()) );
  mEdit->insertItem( i18n("&Unselect"), 
		     editor(), SLOT(unselect()) );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Goto..."),
		     editor(), SLOT(gotoOffset()), CTRL+Key_G );
  mEdit->insertItem( i18n("&Find..."),
		     editor(), SLOT(find()), keys->find() );
  mEdit->insertItem( i18n("Find &Again"),
		     editor(), SLOT(findAgain()), Key_F3 );
  mEdit->insertItem( i18n("Find and R&eplace..."),
		     editor(), SLOT(replace()), keys->replace() );
  mEdit->insertItem( i18n("Insert pattern..."),
		     editor(), SLOT(insertPattern()), CTRL+Key_Insert );

  mEditSpecialCopyText1MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&primary field)"),
			    editor(), SLOT(copyPrimaryFormatted()) );
  mEditSpecialCopyText2MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&all fields)"),
			    editor(), SLOT(copyAllFormatted()) );
  mEditSpecial->insertSeparator();
  mEditSpecial->insertItem( i18n("Paste into new &file"),
			    editor(), SLOT(pasteNewFile()) );
  mEditSpecial->insertItem( i18n("Paste into new &window"),
			    this, SLOT(pasteNewWindow()) );

  mView->setCheckable( TRUE );
  mViewHexMB      = mView->insertItem( i18n("&Hexadecimal"), 
		     view_hexadecimal );
  mViewOctalMB    = mView->insertItem( i18n("&Octal"), 
		     view_octal );
  mViewBinaryMB   = mView->insertItem( i18n("&Binary"), 
		     view_binary );
  mViewTextOnlyMB = mView->insertItem( i18n("&Text only"), 
		     view_text );
  mView->insertSeparator();
  mViewOffsetColumnMB = mView->insertItem( i18n("Show o&ffset column"), 
		     view_offset_area );
  mViewTextAreaMB = mView->insertItem( i18n("Show t&ext area"), 
		     view_text_area );
  mView->insertSeparator();
  mViewOffsetAsDecimalMB = mView->insertItem( i18n("Off&set as decimal"), 
		     view_offset_as_decimal );
  mView->insertSeparator();
  mViewBigCaseDataMB = mView->insertItem( i18n("&Upper case (data)"), 
					  view_case_data );
  mViewBigCaseOffsetMB = mView->insertItem( i18n("U&pper case (offset)"),
					    view_case_offset );

  mView->insertSeparator();
  mView->insertItem( i18n("&Document encoding"), mEncoding );

  connect( mView, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));

  mTools->insertItem( i18n("&Extract strings..."),
		      editor(), SLOT(strings()) );
  mTools->insertItem( i18n("&Record viewer..."),
		      editor(), SLOT(recordView()) );
  mTools->insertItem( i18n("&Binary filters..."),
		      editor(), SLOT(filter()) );
  mTools->insertSeparator();
  mTools->insertItem( i18n("&Statistics..."),
		      editor(), SLOT(statistics()) );
  

  mBookmark->insertItem( i18n("&Add bookmark"), 0, 0 );
  mBookmark->setAccel( CTRL+Key_B, 0 );
  mBookmark->insertItem( i18n("&Replace bookmark"), 1, 1 );
  mBookmark->setAccel( CTRL+Key_E, 1 );
  mBookmark->insertItem( i18n("R&emove bookmark"), 2, 2 );
  mBookmark->setAccel( CTRL+Key_U, 2 );
  mBookmark->insertItem( i18n("Re&move all"), 3, 3 );
  connect( mBookmark, SIGNAL(activated (int)), SLOT( bookmarkMenuCB (int) ));
  
  mOptions->setCheckable( TRUE );
  mOptions->insertItem( i18n("&General preferences..."),
		        option_edit );
  mOptions->insertItem( i18n("&Favorites..."),
		        option_favorites );
  mOptions->insertSeparator();
  mOptionToolBarMB    = mOptions->insertItem( i18n("Show &tool bar"), 
		        option_toolbar );
  mOptionConversionMB = mOptions->insertItem( i18n("Show &conversion field"), 
		        option_conversion );
  mOptionStatusBarMB  = mOptions->insertItem( i18n("Show &status bar"), 
		        option_statusbar );

  mOptions->insertSeparator();
  mOptions->insertItem( i18n("&Save configuration"), this,
			SLOT(writeConfiguration()) );
  connect( mOptions, SIGNAL(activated (int)), SLOT( optionMenuCB (int) ));

  //
  // HACK 1999-05-15-ES (KDE-1.1)
  // Here comes a little hack. The 'Document' menu does not contain any 
  // entries by default. The entries will be added later and shall contain
  // an accelerator (Ctrl+<number=1-9>). The menu entry is created once a
  // document is installed, but without the hack, no accelerator is created.
  // Experimenting has shown that the accelaretor becomes visible if the 
  // menubar goes from "top" to "floating". If I remove all entries, and then
  // add one again it will get the accelerator. So that is what I do here.
  // I add one entry and remove it once the 'Document' menu is inserted.
  // (See addDocument() below) 
  //
  mDocuments->insertItem( "dummy", 0, 0 );
  mDocuments->setAccel( CTRL+Key_1, 0 );

  //  
  // Define menu entries in the menu bar.
  //
  menuBar()->insertItem( i18n("&File"), mFile );
  menuBar()->insertItem( i18n("&Edit"), mEdit );
  menuBar()->insertItem( i18n("&View"), mView );
  menuBar()->insertItem( i18n("&Tools"), mTools );
  menuBar()->insertItem( i18n("&Bookmarks"), mBookmark );
  menuBar()->insertItem( i18n("&Documents"), mDocuments );
  menuBar()->insertItem( i18n("&Options"), mOptions );
  menuBar()->insertSeparator();
  menuBar()->insertItem( i18n("&Help"), mHelp );

  //
  // The dummy has now served its purpose (see above)
  //
  mDocuments->removeItemAt( 0 );

  //
  // Make the toolbar
  //
  QString pixmapPath;
  KIconLoader &loader = *kapp->getIconLoader();
  QPixmap pixmap;

  pixmap = loader.loadIcon( "filenew.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(newFile()), true,
			    i18n("New file") );
  pixmap = loader.loadIcon( "fileopen.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(open()), true,
			    i18n("Open file") );
  pixmap = loader.loadIcon( "reload.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(reload()), true,
			    i18n("Reload file") );
  pixmap = loader.loadIcon( "filefloppy.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(save()), true,
			    i18n("Save file") );
  pixmap = loader.loadIcon( "fileprint.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(print()), true,
			    i18n("Print file") );
  pixmap = loader.loadIcon( "search.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(find()), true,
			    i18n("Find") );
  pixmap = loader.loadIcon( "next.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(findNext()), true,
			    i18n("Find next") );
  pixmap = loader.loadIcon( "prev.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(findPrevious()), true,
			    i18n("Find previous") );
  pixmap = loader.loadIcon( "editcopy.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Copy, SIGNAL(clicked()),
			    editor(), SLOT(copy()), true,
			    i18n("Copy") );
  pixmap = loader.loadIcon( "editpaste.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(paste()), true,
			    i18n("Paste") );
  pixmap = loader.loadIcon( "editcut.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Cut, SIGNAL(clicked()),
			    editor(), SLOT(cut()), true,
			    i18n("Cut") );
  pixmap = loader.loadIcon( "undo.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Undo, SIGNAL(clicked()),
			    editor(), SLOT(undo()), true,
			    i18n("Undo") );
  pixmap = loader.loadIcon( "redo.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_Redo, SIGNAL(clicked()),
			    editor(), SLOT(redo()), true,
			    i18n("Redo") );
  /*
  pixmap = loader.loadIcon( "flag.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(benchmark()), true,
			    i18n("Benchmark") );
  */

  pixmap = loader.loadIcon( "lock.xpm");
  toolBar(0)->insertButton( pixmap, toolbar_rw, SIGNAL(clicked()),
			    editor(), SLOT(toggleWriteProtection()), true,
			    i18n("Toggle write protection") );
  toolBar(0)->alignItemRight( toolbar_rw );


  pixmap = loader.loadIcon( "stop.xpm" );
  toolBar(0)->insertButton( pixmap, toolbar_cancelOperation, SIGNAL(clicked()),
			    editor(), SLOT(stop()), true,
			    i18n("Cancel operation") );

  //
  // Prepare the status bar. It will display (from right) the offset
  // in the file, Save state ( a pixmap), Insert state, general messages
  //

  statusBar()->setInsertOrder( KStatusBar::RightToLeft );
  statusBar()->insertItem( i18n("RW"), 0 );
  statusBar()->insertItem( i18n("Bit: F"), 1 );
  statusBar()->insertItem( i18n("Offset: FFFFFFFFFF"), 2 );
  statusBar()->insertItem( i18n("Size: FFFFFFFFFF"), 3 );
  statusBar()->insertItem( i18n("OVR"), 4 );
  statusBar()->insertItem( i18n("M"), 5 );
  statusBar()->insertItem( i18n("Default encoding"), 6 );

  KNewProgress *mProgess = new KNewProgress(statusBar(), "progress" );
  if( mProgess == 0 ) { return; }
  mProgess->setTextEnabled( false );
  mProgess->setFrameStyle( QFrame::Panel|QFrame::Sunken );
  mProgess->setLineWidth( 1 );
  statusBar()->insertWidget( mProgess, 10, 7 ); 
  mProgess->setRange( 0, 100 ); // 0 is off
 
  connect( editor(), SIGNAL( setProgress( int ) ),
	   mProgess, SLOT( setValue( int ) ) );
  connect( editor(), SIGNAL( setProgress( int, int ) ),
	   mProgess, SLOT( setValue( int, int ) ) );
  connect( editor(), SIGNAL( enableProgressText( bool ) ),
	   mProgess, SLOT( setTextEnabled( bool ) ) );
  connect( editor(), SIGNAL( setProgressText( const QString & ) ),
	   mProgess, SLOT( setText( const QString & ) ) );
  
  statusBar()->setAlignment( 0, AlignCenter );
  statusBar()->setAlignment( 4, AlignCenter );
  statusBar()->setAlignment( 5, AlignCenter );
  statusBar()->changeItem( i18n("Bit:"), 1 );
  statusBar()->changeItem( i18n("Offset:"), 2 );
  statusBar()->changeItem( i18n("Size:"), 3 );
  statusBar()->changeItem( "", 5 );
  statusBar()->changeItem( "", 6 );

  connect( statusBar(), SIGNAL(pressed( int )), 
	   this, SLOT(statusBarPressed( int )) );
  connect( mProgess, SIGNAL(pressed()), 
	   editor(), SLOT(stop()) );

  //
  // Read configuration from file. We will only allow files to 
  // be automatically opened if this in the first window.
  //
  readConfiguration();
  initialize( mWindowList.count() == 1 ? true : false );
  setStartupGeometry();

  mValid = true;
}


KHexEdit::~KHexEdit()
{
  delete mManager;
}


void KHexEdit::initialize( bool openFiles )
{
  mUndoState = UNDO_OK;   // Reset in function below
  setUndoState( 0 ); 

  mSelectionValid = true; // Reset in function below
  setSelectionState( false );

  operationChanged( false );

  editor()->initialize();
 
  mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
  if( mShowToolBar == true )
  {
    enableToolBar(KToolBar::Show);
  }
  else
  {
    enableToolBar(KToolBar::Hide);
  }

  mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
  if( mShowStatusBar == true )
  {
    enableStatusBar(KStatusBar::Show);
  }
  else
  {
    enableStatusBar(KStatusBar::Hide);
  }

  mOptions->setItemChecked( mOptionConversionMB, mShowConversion );
  mManager->setShowUtil( mShowConversion );

  if( mRecent->count() > 0 && openFiles == true )
  {
    if( editor()->openFile() == SDisplayMisc::allRecent )
    {
      for( uint i=0; i < mRecent->count(); i++ )
      {
	mStartupFileList.append( mRecent->text(i) );
      }
      startTimer( 100 );
    }
    else if( editor()->openFile() == SDisplayMisc::mostRecent )
    {
      mStartupFileList.append( mRecent->text(0) );
      startTimer( 100 );
    }
  }

}


void KHexEdit::timerEvent( QTimerEvent *e )
{
  killTimers();
  open( mStartupFileList );
  
  //
  // We use a deep copy so the elements should be released now. 
  //
  while( mStartupFileList.count() > 0 )
  {
    mStartupFileList.removeFirst();
  }

}










void KHexEdit::statusBarPressed( int id )
{
  if( id == 0 )
  {
    SDisplayInputMode &input = editor()->inputMode();
    input.readOnly = input.readOnly == true ? false : true;
    textView()->setInputMode( input );
  }
  else if( id == 2 )
  {
    SDisplayLayout &layout = editor()->layout();
    if( layout.offsetMode == SDisplayLayout::hexadecimal )
    {
      layout.offsetMode = SDisplayLayout::decimal;
    }
    else
    {	
      layout.offsetMode = SDisplayLayout::hexadecimal;
    }
    textView()->setLayout( layout );
  }
}



void KHexEdit::open( QStrList &fileList )
{
  //
  // We load in reverse to allow first document in the list to
  // become the active one. 
  //
  for( char *s = fileList.last(); s != 0; s = fileList.prev() )
  {
    QString url( s );
    editor()->open( url );
  }
}

void KHexEdit::addRecentFile( QString &fileName )
{
  if( mBlockRecentUpdate == true )
  {
    return;
  }

  if( fileName.isEmpty() == true )
  {
    return;
  }
  
  if( fileName.contains( i18n( "Untitled" ), false ) )
  {
    return;
  }

  uint recentCount = mRecent->count();
  uint i;

  for( i=0; i < recentCount; i++ )
  {
    const char *text = mRecent->text(i);
    if( text == 0 )
    {
      continue;
    }
 
    if( fileName == text )
    { 
      //return;
      mRecent->removeItemAt(i);
    }
  }
  
  if( mRecent->count() >= 5 ) 
  {
    mRecent->removeItemAt( 4 );
  }

  mRecent->insertItem( fileName, 0, 0 );
  for( i = 0; i < 5; i++ ) 
  {
    mRecent->setId( i, i );
  }
}


void KHexEdit::removeRecentFile( const QString &fileName )
{
  if( fileName.isEmpty() == true )
  {
    return;
  }
  
  if( fileName.contains( i18n( "Untitled" ), false ) )
  {
    return;
  }

  uint recentCount = mRecent->count();
  uint i;

  for( i=0; i < recentCount; i++ )
  {
    const char *text = mRecent->text(i);
    if( text == 0 )
    {
      continue;
    }
 
    if( fileName == text )
    { 
      mRecent->removeItemAt(i);
    }
  }
  for( i = 0; i < 5; i++ ) 
  {
    mRecent->setId( i, i );
  }
}


void KHexEdit::renameRecentFile( QString &curName, QString &newName )
{
  if( curName.contains( i18n( "Untitled" ), false ) )
  {
    addRecentFile( newName );
  }
  else
  {
    for( uint i=0; i < mRecent->count(); i++ )
    {
      const char *text = mRecent->text(i);
      if( text == 0 ) { continue; }
 
      if( curName == text )
      { 
	mRecent->changeItem( newName.data(), i );
	break;
      }
    }
  }
}




void KHexEdit::recentFile( int id )
{
  const char *text = mRecent->text(id);
  if( text != 0 )
  {
    QString url( text );
    editor()->open( url );
  }
}


void KHexEdit::newWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg;
    msg += i18n( "Can not create new window." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("New window" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0);
    return;
  }
  app->show();
}



void KHexEdit::pasteNewWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg;
    msg += i18n( "Can not create new window." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("New window" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0);
    return;
  }
  app->show();
  app->editor()->pasteNewFile();
}



void KHexEdit::closeWindow( void )
{
  close();
}


void KHexEdit::closeProgram( void )
{
  if( mWindowList.count() > 1 )
  {
    for( KHexEdit *w = mWindowList.first(); w != 0; w = mWindowList.next() )
    {
      if( w->editor()->modified() == true )
      {
	QString msg;
	msg += i18n( "There are windows with unsaved modified documents." );
	msg += "\n";
	msg += i18n( "If you quit now, these modifications will be lost." );
	msg += "\n\n";
	msg += i18n( "Proceed?" );
      
	int reply = QMessageBox::warning( 
	  topLevelWidget(),
	  i18n( "Warning" ), 
	  msg.data(),
	  i18n( "&Yes" ),
	  i18n( "&No" ),
	  "",
	  1, 1 );
	if( reply == 0 ) // Yes
	{
	  break;
	}
	else
	{
	  return;
	}
      }
    }

    //
    // Just to save some data
    //
    queryExit();

    //
    // Now we need to destroy the manager widget. The editor widget 
    // (which is a child of the manager widget) will delete some
    // dialogs when destroyed. These dialogs have no parent (parent=0)
    // and sometimes their destructors are executed before the editor
    // widget are destroyed (unless we do as below). That's bad.
    //
    for( KHexEdit *w = mWindowList.first(); w != 0; w = mWindowList.next() )
    {
      delete w->mManager; w->mManager = 0;
    }
    kapp->quit();

  }
  else
  {
    closeWindow();
  }
 
}
    

bool KHexEdit::queryClose( void )
{
  mBlockRecentUpdate = true;
  bool success = editor()->closeAll();
  if( success == true )
  {
    mWindowList.removeRef( this );
  }
  mBlockRecentUpdate = success;
  return( success );
}


//
// We will always return true here. 
//
bool KHexEdit::queryExit( void )
{
  mBlockRecentUpdate = true;
  writeConfiguration();
  return( true );
}



void KHexEdit::readConfiguration( void )
{
  KConfig &config = *kapp->getConfig();
  readConfiguration( config );
}


void KHexEdit::writeConfiguration( void )
{
  KConfig &config = *kapp->getConfig();
  writeConfiguration( config );
}


void KHexEdit::writeConfiguration( KConfig &config )
{
  config.setGroup("General Options" );
  config.writeEntry("Width", width() );
  config.writeEntry("Height", height() );
  config.writeEntry("ShowToolBar", mShowToolBar );
  config.writeEntry("ShowStatusBar", mShowStatusBar );
  config.writeEntry("ShowUtility", mShowConversion );

  for( uint i=0; i < 5; i++ )
  {
    const char *text = i < mRecent->count() ? mRecent->text(i) : "";
    if( text != 0 )
    {
      char buf[20];
      sprintf( buf, "Recent%d", i+1 );
      config.writeEntry( buf, text );
    }
  }

  util()->writeConfiguration( config );
  editor()->writeConfiguration( config );
}


void KHexEdit::readConfiguration( KConfig &config )
{
  config.setGroup("General Options");
  mShowToolBar    = config.readBoolEntry( "ShowToolBar", true );
  mShowStatusBar  = config.readBoolEntry( "ShowStatusBar", true );
  mShowConversion = config.readBoolEntry( "ShowUtility", true );

  for( uint i = 0; i < 5; i++ ) 
  {
    char buf[20];
    sprintf( buf, "Recent%d", i + 1 );
    QString fileName = config.readEntry( buf );
    if( fileName.isEmpty() == false )
    { 
      mRecent->insertItem( fileName );
    }
  }
 
  util()->readConfiguration( config );
  editor()->readConfiguration( config );
}



void KHexEdit::setStartupGeometry( void )
{
  KConfig &config = *kapp->getConfig();
  config.setGroup("General Options");
  int w = config.readNumEntry( "Width", -1 );
  int h = config.readNumEntry( "Height", -1 );
  if( w < 10 || h < 10 )
  {
    setGeometry( 0, 0, editor()->defaultTextWidth(), 500 );
  }
  else
  {
    setGeometry( 0, 0, w, h );
  }
}



void KHexEdit::operationChanged( bool state )
{
  toolBar()->setItemEnabled( toolbar_cancelOperation, state );
  mFile->setItemEnabled( mFileCancelPrintMB, state );
}





void KHexEdit::cursorChanged( SCursorState &state )
{
  QString bit( i18n("Bit:") );
  QString offset( i18n("Offset:") );

  if( state.valid == false )
  {
    statusBar()->changeItem( bit.data(), 1 );
    statusBar()->changeItem( offset.data(), 2 );
  }
  else
  {
    SDisplayLayout &layout = editor()->layout();
    char buf[20];
    
    sprintf( buf, " %u", state.cell );
    bit += buf;
    statusBar()->changeItem( bit.data(), 1 );

    if( layout.offsetMode == SDisplayLayout::hexadecimal )
    {
      if( layout.offsetUpperCase == false )
      {
	sprintf( buf, " %04x:%04x", state.offset>>16, state.offset&0x0000FFFF);
      }
      else
      {
	sprintf( buf, " %04X:%04X", state.offset>>16, state.offset&0x0000FFFF);
      }
    }
    else
    {
      sprintf( buf, " %010u", state.offset );
    }

    offset += buf;
    statusBar()->changeItem( offset.data(), 2 );
  }
  
  setUndoState( state.undoState );
  setSelectionState( state.selectionValid );
}


void KHexEdit::fileModified( SFileState &state )
{
  if( state.valid == true )
  {
    QString size;
    size.sprintf( "%s %u", i18n("Size:"), state.size );
    statusBar()->changeItem( size.data(), 3 );
    statusBar()->changeItem( state.modified == true ? "!" : "", 5 );
  }
  else
  {
    statusBar()->changeItem( "", 3 );
    statusBar()->changeItem( "", 5 );
  }
}



void KHexEdit::layoutChanged( const SDisplayLayout &layout )
{
  if( layout.primaryMode == SDisplayLayout::hexadecimal ) 
  {
    mView->setItemChecked( mViewHexMB, true );
    mEditorPopup->setItemChecked( mPopHexMB, true );
  }
  else
  {
    mView->setItemChecked( mViewHexMB, false );
    mEditorPopup->setItemChecked( mPopHexMB, false );
  }

  if( layout.primaryMode == SDisplayLayout::octal )
  { 
    mView->setItemChecked( mViewOctalMB, true );
    mEditorPopup->setItemChecked( mPopOctalMB, true );
  }
  else
  {
    mView->setItemChecked( mViewOctalMB, false );
    mEditorPopup->setItemChecked( mPopOctalMB, false );
  }

  if( layout.primaryMode == SDisplayLayout::binary )
  { 
    mView->setItemChecked( mViewBinaryMB, true );
    mEditorPopup->setItemChecked( mPopBinaryMB, true );
  }
  else
  {
    mView->setItemChecked( mViewBinaryMB, false );
    mEditorPopup->setItemChecked( mPopBinaryMB, false );
  }
  
  if( layout.primaryMode == SDisplayLayout::textOnly )
  { 
    mView->setItemChecked( mViewTextOnlyMB, true );
    mView->setItemEnabled( mViewTextAreaMB, false );
    mEditorPopup->setItemChecked( mPopTextOnlyMB, true );
  }
  else
  {
    mView->setItemChecked( mViewTextOnlyMB, false );
    mView->setItemEnabled( mViewTextAreaMB, true );
    mEditorPopup->setItemChecked( mPopTextOnlyMB, false );
  }

  mView->setItemChecked( mViewOffsetColumnMB, layout.offsetVisible );
  //mView->setItemEnabled( mViewBigCaseOffsetMB, layout.offsetVisible );

  if( layout.offsetMode == SDisplayLayout::hexadecimal )
  {
    mView->setItemChecked( mViewOffsetAsDecimalMB, false );
  }
  else
  {
    mView->setItemChecked( mViewOffsetAsDecimalMB, true );
  }

  if( layout.secondaryMode == SDisplayLayout::hide )
  {
    mView->setItemChecked( mViewTextAreaMB, false );
  }
  else
  {
    mView->setItemChecked( mViewTextAreaMB, true );
  }

  mView->setItemChecked( mViewBigCaseOffsetMB, layout.offsetUpperCase );
  mView->setItemChecked( mViewBigCaseDataMB, layout.primaryUpperCase );
}


void KHexEdit::inputModeChanged( const SDisplayInputMode &mode )
{
  mFilePermission->setItemChecked( mPermissionReadOnlyMB, mode.readOnly );
  mFilePermission->setItemChecked( mPermissionResizeMB, mode.allowResize );
  mEditorPopup->setItemChecked( mPopReadOnlyMB, mode.readOnly );
  mEditorPopup->setItemChecked( mPopResizeMB, mode.allowResize );
  statusBar()->changeItem( mode.readOnly == true ? i18n("R") : i18n("RW"), 0 );
  
  QPixmap pixmap;
  KIconLoader &ic = *kapp->getIconLoader();
  if( mode.readOnly == false )
  {
    QString path( kapp->localkdedir());
    path += "/share/icons/mini/hexwrite.xpm";
    pixmap = ic.loadIcon( path );
  }
  else
  {
    pixmap = ic.loadIcon( "lock.xpm");
  }
  KToolBarButton *tb =  toolBar(0)->getButton(toolbar_rw);
  tb->setPixmap( pixmap );


}



void KHexEdit::bookmarkChanged( QList<SCursorOffset> &list )
{
  mBookmark->clear();
  mBookmarkPopup->clear();

  mBookmark->insertItem( i18n("&Add bookmark"), 0, 0 );
  mBookmark->setAccel( CTRL+Key_B, 0 );
  mBookmark->insertItem( i18n("&Replace bookmark"), 1, 1 );
  mBookmark->setAccel( CTRL+Key_E, 1 );
  mBookmark->insertItem( i18n("R&emove bookmark"), 2, 2 );
  mBookmark->setAccel( CTRL+Key_U, 2 );
  mBookmark->insertItem( i18n("Re&move all"), 3, 3 );

  if( list.count() == 0 )
  {
    return;
  }

  uint entry = 4;
  mBookmark->insertSeparator( entry++ );

  QString str;
  for( uint i=0; i < list.count(); i++ )
  {

    const SCursorOffset *p = list.at(i);
    if( p == 0 ) { continue; }

    str.sprintf( "%s: %04X:%04X", i18n("Offset"), p->offset>>16, 
		 p->offset&0x0000FFFF );
    mBookmark->insertItem( str.data(), i+entry );
    mBookmark->setId( i+entry, i+entry );
    
    int key = acceleratorNumKey( i );
    if( key > 0 )
    {
      mBookmark->setAccel( ALT+key, i+entry );
    }

    str.sprintf( "[%u] %s: %04X:%04X", i+1, i18n("Offset"),
		 p->offset>>16, p->offset&0x0000FFFF );
    mBookmarkPopup->insertItem( str.data() );
  }

}


void KHexEdit::fileActive( QString &url )
{
  if( url.isNull() == true )
  {
    setCaption( kapp->getCaption() );
  }
  else
  {
    QString str( url );
    str += " (";
    str += kapp->getCaption();
    str += ")";
    setCaption( str.data() );

    addDocument( url );
    addRecentFile( url );
  }
}


void KHexEdit::fileRename( QString &curName, QString &newName )
{
  if( newName.isNull() == true )
  {
    return;
  }

  renameDocument( curName, newName );
  renameRecentFile( curName, newName );

  QString str( newName );
  str += " (";
  str += kapp->getCaption();
  str += ")";
  setCaption( str.data() );
}



void KHexEdit::fileClosed( QString &url )
{
  removeDocument( url );
}



void KHexEdit::editMode( CHexBuffer::EEditMode editMode )
{
  statusBar()->changeItem( editMode == CHexBuffer::edit_overwrite ? 
			   i18n("OVR") : i18n("INS"), 4 );
}


void KHexEdit::encodingChanged( const SEncodeState &state )
{    
  bool mode = state.mode == CConversion::cnvDefault ? true : false;
  mEncoding->setItemChecked( mEncodingDefaultMB, mode );
  mode = state.mode == CConversion::cnvEbcdic ? true : false;
  mEncoding->setItemChecked( mEncodingEbcdicMB, mode );
  mode = state.mode == CConversion::cnvUsAscii ? true : false;
  mEncoding->setItemChecked( mEncodingUsAsciiMB, mode );

  statusBar()->changeItem( state.name == 0 ? "" : state.name, 6 );
}



void KHexEdit::textWidth( uint width )
{
  setGeometry( x(), y(), editor()->defaultTextWidth(), height() );
}



void KHexEdit::dropAction( KDNDDropZone *dropZone )
{
  QStrList &list = dropZone->getURLList();
  open( list );
}


void KHexEdit::viewMenuCB( int id )
{
  SDisplayLayout &layout = editor()->layout();
  SDisplayLine &line = editor()->line();

  switch( id )
  {
    case view_hexadecimal:
      layout.primaryMode   = SDisplayLayout::hexadecimal;
      layout.lineSize      = line.lineSize[ SDisplayLine::hexadecimal ];
      layout.columnSize    = line.columnSize[ SDisplayLine::hexadecimal ];
      textView()->setLayout( layout );
    break;

    case view_octal:
      layout.primaryMode   = SDisplayLayout::octal;
      layout.lineSize      = line.lineSize[ SDisplayLine::octal ];
      layout.columnSize    = line.columnSize[ SDisplayLine::octal ];
      textView()->setLayout( layout );
    break;

    case view_binary:
      layout.primaryMode   = SDisplayLayout::binary;
      layout.lineSize      = line.lineSize[ SDisplayLine::binary ];
      layout.columnSize    = line.columnSize[ SDisplayLine::binary ];
      textView()->setLayout( layout );
    break;

    case view_text:
      layout.primaryMode   = SDisplayLayout::textOnly;
      layout.lineSize      = line.lineSize[ SDisplayLine::textOnly ];
      layout.columnSize    = line.columnSize[ SDisplayLine::textOnly ];
      textView()->setLayout( layout );
    break;

    case view_offset_area:
      layout.offsetVisible = layout.offsetVisible == true ? false : true;
      textView()->setLayout( layout );
    break;

    case view_text_area:
      if( layout.secondaryMode == SDisplayLayout::hide )
      {
	layout.secondaryMode = SDisplayLayout::textOnly;
      }
      else
      {
	layout.secondaryMode = SDisplayLayout::hide;
      }
      textView()->setLayout( layout );
    break;

    case view_offset_as_decimal:
      if( layout.offsetMode == SDisplayLayout::hexadecimal )
      {
	layout.offsetMode = SDisplayLayout::decimal;
      }
      else
      {	
	layout.offsetMode = SDisplayLayout::hexadecimal;
      }
      textView()->setLayout( layout );
    break;

    case view_case_data:
      if( layout.primaryUpperCase == true )
      {	
	layout.primaryUpperCase = false;
      }
      else
      {
	layout.primaryUpperCase = true;
      }
      textView()->setLayout( layout );
    break;

    case view_case_offset:
      if( layout.offsetUpperCase == true )
      {
	layout.offsetUpperCase = false;
      }
      else
      {
	layout.offsetUpperCase = true;
      }
      textView()->setLayout( layout );
    break;

    case view_lock_width:
      layout.lockLine = layout.lockLine == True ? False : True;
      textView()->setLayout( layout );
    break;

    case permission_readonly:
      editor()->toggleWriteProtection();
    break;

    case permission_allowResize:
    {
      SDisplayInputMode &input = editor()->inputMode();
      input.allowResize = input.allowResize == true ? false : true;
      textView()->setInputMode( input );
    }
    break;
  }
}


void KHexEdit::encodingMenuCB( int id )
{
  switch( id )
  {
    case encoding_default:
      editor()->encode( CConversion::cnvDefault );
    break;

    case encoding_us_ascii:
      editor()->encode( CConversion::cnvUsAscii );
    break;

    case encoding_ebcdic:
      editor()->encode( CConversion::cnvEbcdic );
    break;
  }
}


void KHexEdit::bookmarkMenuCB( int id )
{
  if( id == 0 )
  {
    editor()->addBookmark();
  }
  else if( id == 1 )
  {
    editor()->replaceBookmark();
  }
  else if( id == 2 )
  {
    editor()->removeBookmark( false );
  }
  else if( id == 3 )
  {
    editor()->removeBookmark( true );
  }
  else if( id >= 5 )
  {
    editor()->gotoBookmark( id - 5 );
  }
}


void KHexEdit::bookmarkPopupCB( int id )
{
  editor()->gotoBookmark( id );
}


void KHexEdit::optionMenuCB( int id )
{
  switch( id )
  {
    case option_toolbar:
      mShowToolBar = mShowToolBar == true ? false : true;
      mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
      if( mShowToolBar == true )
      {
	enableToolBar(KToolBar::Show);
      }
      else
      {
	enableToolBar(KToolBar::Hide);
      }
    break;

    case option_statusbar:
      mShowStatusBar = mShowStatusBar == true ? false : true;
      mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
      if( mShowStatusBar == true )
      {
	enableStatusBar(KStatusBar::Show);
      }
      else
      {
	enableStatusBar(KStatusBar::Hide);
      }
    break;

    case option_conversion:
      mShowConversion = mManager->showUtil() == true ? false : true;
      mOptions->setItemChecked( mOptionConversionMB, mShowConversion );
      mManager->setShowUtil( mShowConversion );
    break;

    case option_edit:
      editor()->options();
    break;

    case option_favorites:
      editor()->favorites();
    break;
  }
}





void KHexEdit::setUndoState( uint undoState ) 
{
  if( mUndoState != undoState )
  {
    mUndoState = undoState;
    mEdit->setItemEnabled( mEditUndoMB, mUndoState & UNDO_OK );
    mEdit->setItemEnabled( mEditRedoMB, mUndoState & REDO_OK );
    toolBar()->setItemEnabled( toolbar_Undo, mUndoState & UNDO_OK );
    toolBar()->setItemEnabled( toolbar_Redo, mUndoState & REDO_OK );
  }
}


void KHexEdit::setSelectionState( bool selectionValid )
{
  if( mSelectionValid != selectionValid )
  {
    mSelectionValid = selectionValid;
    mEdit->setItemEnabled( mEditCopyMB, mSelectionValid );
    mEdit->setItemEnabled( mEditCutMB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText1MB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText2MB, mSelectionValid );

    toolBar()->setItemEnabled( toolbar_Copy, mSelectionValid );
    toolBar()->setItemEnabled( toolbar_Cut, mSelectionValid );
  }
}



void KHexEdit::documentMenuCB( int id )
{
  QString *entry = mDocumentList.at(id);
  if( entry != 0 )
  {
    editor()->selectDocument( *entry );
  }
}



void KHexEdit::addDocument( QString &fileName )
{
  uint documentCount = mDocumentList.count();
  for( uint i=0; i < documentCount; i++ )
  {
    QString *entry = mDocumentList.at(i);
    if( entry == 0 ) { continue; }

    if( fileName == *entry )
    {
      return;
    }
  }

  QString *newEntry = new QString( fileName.data() );
  if( newEntry == 0 ) { return; }

  mDocumentList.append( newEntry );

  mDocuments->insertItem( fileName, documentCount, documentCount );
  int key = acceleratorNumKey( documentCount );
  if( key > 0 )
  {
    mDocuments->setAccel( CTRL+key, documentCount );
  }
}



void KHexEdit::removeDocument( QString &fileName )
{
  uint documentCount = mDocumentList.count();
  for( uint i=0; i < documentCount; i++ )
  {
    QString *entry = mDocumentList.at(i);
    if( entry == 0 ) { continue; }

    if( fileName == *entry )
    {
      mDocumentList.remove( i );
      mDocuments->removeItemAt( i );

      for( i = 0; i < documentCount; i++ ) 
      {
	mDocuments->setId( i, i );
	int key = acceleratorNumKey( i );
	if( key > 0 )
	{
	  mDocuments->setAccel( CTRL+key, i );
	}
      }
      return;
    }
  }
}



void KHexEdit::renameDocument( QString &curName, QString &newName )
{
  uint documentCount = mDocumentList.count();
  for( uint i=0; i < documentCount; i++ )
  {
    QString *entry = mDocumentList.at(i);
    if( entry == 0 ) { continue; }

    if( curName == *entry )
    {
      (*entry).truncate(0);
      (*entry) = newName.data();
      mDocuments->changeItem( newName.data(), i );
      return;
    }
  }
}


